extends Spatial

# stream for the test sound
var m_Stream

# reusable AudioStreamPlayer3D
var m_Player

# timer used to give regular play sounds
var m_Timer = 0

# which method currently being used
var m_Method = 1

func _ready():
	# preload the sound and get the reusable player
	m_Stream = load("res://bleep.wav")
	m_Player = $AudioStreamPlayer3D

func _process(delta):
	# allow the user to change between the two methods and display a label showing this
	if Input.is_action_just_pressed("ui_accept"):
		var label = find_node("Label_Method", true, false)
		
		if m_Method == 1:
			m_Method = 2
			label.text = "Method 2 - New AudioStreamPlayer each play"
		else:
			m_Method = 1	
			label.text = "Method 1 - Reuse AudioStreamPlayer"
	


func _physics_process(delta):
	# attempt to play the test sound at regular intervals
	# (just below once per second, the sound is 1 second long)
	if (m_Timer % 70) == 0:
		
		# choose a random position (for pan)
		var x = -50 + (randi() % 100)
		
		# play sound using chosen method
		if m_Method == 1:
			PlaySound_Method1(x, randf())
		else:
			PlaySound_Method2(x, randf())

	m_Timer += 1

func _DBFromVolume(vol : float)->float:
	return 8.69 * log(vol)


func PlaySound_Method1(var x, var volume : float):
	# can only play one sound at a time
	if m_Player.is_playing():
		return

	m_Player.translation = Vector3(x, 0, 0)
	m_Player.stream = m_Stream
	m_Player.set_max_db(_DBFromVolume(volume))
	m_Player.play()	
	pass


func PlaySound_Method2(var x, var volume : float):
	# create a new AudioStreamPlayer each time
	var player : AudioStreamPlayer3D = AudioStreamPlayer3D.new()
	
	# add to the Main spatial
	add_child(player)
	
	player.translation = Vector3(x, 0, 0)
	player.stream = m_Stream
	player.set_max_db(_DBFromVolume(volume))
	
	# make a signal when the sound is finished playing
	# to delete the temporary AudioStreamPlayer
	player.connect("finished", player, "queue_free")
	
	player.play()